@@ -36,60 +36,26 @@ class FakeFlags(object):
     self.comments = True
     self.dry_run = False
     self.ignore_re = None
-    self.checkout_command = None
     self.safe_headers = False
     self.separate_project_includes = None
-    self.create_cl_if_possible = True
     self.invoking_command_line = 'iwyu.py my_targets'
     self.find_affected_targets = True
     self.keep_iwyu_namespace_format = False
 
-class FakeRunCommand(object):
-  def __init__(self):
-    self.command = None
-    self.args = None
-
-  def RunCommand(self, command, args):
-    self.command = command
-    self.args = args
-
-
-class FakeGetCommandOutputWithInput(object):
-  def __init__(self, stdout):
-    self.stdout = stdout
-    self.command = None
-    self.stdin = None
-
-  def GetCommandOutputWithInput(self, command, stdin):
-    print('GetCommandOutputWithInput(%s, ...)' % command)
-    self.command = command
-    self.stdin = stdin
-    return self.stdout
-
-
-class FakeGetCommandOutputLines(object):
-  def __init__(self):
-    self.stdout_lines_map = {}
-    self.command = None
-    self.args = None
-
-  def GetCommandOutputLines(self, command, args):
-    print('GetCommandOutputLines(%s, %s)' % (command, args))
-    self.command = command
-    self.args = args
-    return self.stdout_lines_map[command]
-
-  def SetCommandOutputLines(self, command, lines):
-    lines = [line + '\n' for line in lines]
-    self.stdout_lines_map[command] = lines
 
 class FixIncludesBase(unittest.TestCase):
   """Does setup that every test will want."""
 
-  def _ReadFile(self, filename):
+  def _ReadFile(self, filename, fileinfo):
     assert filename in self.before_map, filename
     return self.before_map[filename]
 
+  def _ParseFileInfo(self, filename):
+      return fix_includes.FileInfo('\n', 'utf-8')
+
+  def _WriteFile(self, filename, fileinfo, contents):
+      return self.actual_after_contents.extend(contents)
+
   def setUp(self):
     self.flags = FakeFlags()
 
@@ -100,57 +66,20 @@ class FixIncludesBase(unittest.TestCase):
 
     # INPUT: fix_includes._ReadFile takes a filename
     # and returns the contents of filename (as a list).
+    # FileInfo controls encoding details of the file,
+    # wire it to return something that agrees with the
+    # tests.
     fix_includes._ReadFile = self._ReadFile
+    fix_includes.FileInfo.parse = self._ParseFileInfo
 
     # OUTPUT: Instead of writing to file, save full output.
     self.actual_after_contents = []
-    fix_includes._WriteFileContents = \
-        lambda filename, contents: self.actual_after_contents.extend(contents)
-
-    # Stub out OS writeability check to say all files are writeable.
-    fix_includes.os.access = \
-        lambda filename, flags: True
-
-    # Stub out the checkout command; keep a list of all files checked out.
-    self.fake_checkout_command = FakeRunCommand()
-    fix_includes._RunCommand = self.fake_checkout_command.RunCommand
-
-    # Stub out the CL creation command.
-    self.fake_cl_creation_command = (
-        FakeGetCommandOutputWithInput('Change 1234 created.\n'))
-    fix_includes._GetCommandOutputWithInput = (
-        self.fake_cl_creation_command.GetCommandOutputWithInput)
-
-    # Create a generic 'stubber'.
-    self.fake_get_command_output_lines = FakeGetCommandOutputLines()
-    fix_includes._GetCommandOutputLines = (
-        self.fake_get_command_output_lines.GetCommandOutputLines)
-
-    # Stub out the CL template command.
-    self.fake_get_command_output_lines.SetCommandOutputLines(
-        'g4', ['Description:',
-               '\t<enter description here>',
-               '',
-               'Files:',
-               '\tthis_file_should_not_appear_in_the_cl'])
-
-    # Stub out the find_clients_of_files command.
-    self.find_clients_of_files_command = (
-        '/google/data/ro/projects/cymbal/tools/find_clients_of_files.par')
-    self.fake_get_command_output_lines.SetCommandOutputLines(
-        self.find_clients_of_files_command,
-        ['==== 1 targets from 1 packages are affected',
-         '==== targets',
-         '//foo/bar:a'])
+    fix_includes._WriteFile = self._WriteFile
 
     # Stub out stdout
     self.stdout_stub = StringIO()
     fix_includes.sys.stdout = self.stdout_stub
 
-  def SetFindClientsOfFilesOutput(self, output_lines):
-    self.fake_get_command_output_lines.SetCommandOutputLines(
-        self.find_clients_of_files_command, output_lines)
-
   def RegisterFileContents(self, file_contents_map):
     """Parses and stores the given map from filename to file-contents.
 
@@ -231,11 +160,6 @@ class FixIncludesBase(unittest.TestCase):
     if expected_num_modified_files is not None:
       self.assertEqual(expected_num_modified_files, num_modified_files)
 
-  def MakeFilesUnwriteable(self):
-    """Stub out OS writeability check to say all files are NOT writeable."""
-    fix_includes.os.access = \
-        lambda filename, flags: False
-
 
 class FixIncludesTest(FixIncludesBase):
   """End-to-end tests from input file to output file."""
@@ -2895,36 +2819,6 @@ The full include-list for no_key:
                       iwyu_output.splitlines(),
                       self.flags)
 
-  def testNotWriteable(self):
-    """Test that files don't get rewritten if they are not writeable."""
-    infile = """\
-// Copyright 2010
-
-#include <notused.h>
-#include "used.h"
-
-int main() { return 0; }
-"""
-    iwyu_output = """\
-unwritable should add these lines:
-#include <stdio.h>
-#include "used2.h"
-
-unwritable should remove these lines:
-- #include <notused.h>  // lines 3-3
-
-The full include-list for unwritable:
-#include <stdio.h>
-#include "used.h"
-#include "used2.h"
----
-"""
-    self.RegisterFileContents({'unwritable': infile})
-    self.MakeFilesUnwriteable()
-    # No files are written, because they are not writeable.
-    self.ProcessAndTest(iwyu_output, unedited_files=['unwritable'])
-
-
   def testFileSpecifiedOnCommandline(self):
     """Test we limit editing to files specified on the commandline."""
     changed_infile = """\
@@ -3101,7 +2995,6 @@ int main() { return 0; }
                          self.actual_after_contents)
     self.assertEqual(2, num_files_modified)
 
-
   def testSortingIncludesAlreadySorted(self):
     """Tests sorting includes only, when includes are already sorted."""
     infile = """\
@@ -3548,5 +3441,49 @@ namespace ns { namespace ns4 { class Baz; } }
                       ['fix_includes.py', '-s'])
 
 
+class FileInfoTest(unittest.TestCase):
+  """ Unit test for file info detection """
+
+  def testEndingsWindows(self):
+    buf = b'first\r\nsecond\r\nthird\r\n'
+    self.assertEqual('\r\n', fix_includes.FileInfo.guess_linesep(buf))
+
+  def testEndingsUnix(self):
+    buf = b'first\nsecond\nthird\n'
+    self.assertEqual('\n', fix_includes.FileInfo.guess_linesep(buf))
+
+  def testEndingsMixedUnixMajority(self):
+    buf = b'first\nsecond\nsecond-and-a-half\r\nthird\nfourth\r\n'
+    self.assertEqual('\n', fix_includes.FileInfo.guess_linesep(buf))
+
+  def testEndingsMixedWindowsMajority(self):
+    buf = b'first\nsecond\r\nsecond-and-a-half\r\nthird\nfourth\r\n'
+    self.assertEqual('\r\n', fix_includes.FileInfo.guess_linesep(buf))
+
+  def testEndingsMixedTie(self):
+    buf = b'first\nsecond\nthird\r\nfourth\r\n'
+    self.assertEqual(fix_includes.FileInfo.DEFAULT_LINESEP,
+                     fix_includes.FileInfo.guess_linesep(buf))
+
+  def testEncodingASCII(self):
+    buf = b'abcdefgh'
+    self.assertEqual('ascii', fix_includes.FileInfo.guess_encoding(buf))
+
+  def testEncodingUTF8BOM(self):
+    buf = b'\xef\xbb\xbfSomeASCIIButWithTheBOM'
+    self.assertEqual('utf-8', fix_includes.FileInfo.guess_encoding(buf))
+
+  def testEncodingUTF8NoBOM(self):
+    # This is a recurring test input in Swedish, translates to "shrimp sandwich"
+    # and contains all three Swedish exotic characters.
+    buf = b'r\xc3\xa4ksm\xc3\xb6rg\xc3\xa5s'
+    self.assertEqual('utf-8', fix_includes.FileInfo.guess_encoding(buf))
+
+  def testEncodingISO8859_1(self):
+    # Yours truly
+    buf = b'Kim Gr\xe4sman'
+    self.assertEqual('windows-1250', fix_includes.FileInfo.guess_encoding(buf))
+
+
 if __name__ == '__main__':
   unittest.main()
