@@ -262,6 +262,25 @@ bool IsNodeInsideCXXMethodBody(const ASTNode* ast_node) {
   return false;
 }
 
+UseFlags ComputeUseFlags(const ASTNode* ast_node) {
+  UseFlags flags = UF_None;
+
+  if (IsNodeInsideCXXMethodBody(ast_node))
+    flags |= UF_InCxxMethodBody;
+
+  // Definitions of free functions are a little special, because they themselves
+  // count as uses of all prior declarations (ideally we should probably just
+  // require one but it's hard so say which, so we pick all previously seen).
+  // Later IWYU analysis phases do some canonicalization that isn't
+  // necessary/valid for this case, so mark it up for later.
+  if (const auto* fd = ast_node->GetAs<FunctionDecl>()) {
+    if (fd->getKind() == Decl::Function && fd->isThisDeclarationADefinition())
+      flags |= UF_FunctionDfn;
+  }
+
+  return flags;
+}
+
 bool IsNestedClassAsWritten(const ASTNode* ast_node) {
   return (ast_node->IsA<RecordDecl>() &&
           (ast_node->ParentIsA<CXXRecordDecl>() ||
@@ -452,16 +471,16 @@ string PrintableTemplateName(const TemplateName& tpl_name) {
 string PrintableTemplateArgument(const TemplateArgument& arg) {
   std::string buffer;
   raw_string_ostream ostream(buffer);
-  TemplateSpecializationType::PrintTemplateArgumentList(
-      ostream, ArrayRef<TemplateArgument>(arg), DefaultPrintPolicy());
+  printTemplateArgumentList(ostream, ArrayRef<TemplateArgument>(arg),
+                            DefaultPrintPolicy());
   return ostream.str();
 }
 
 string PrintableTemplateArgumentLoc(const TemplateArgumentLoc& arg) {
   std::string buffer;
   raw_string_ostream ostream(buffer);
-  TemplateSpecializationType::PrintTemplateArgumentList(
-      ostream, ArrayRef<TemplateArgumentLoc>(arg), DefaultPrintPolicy());
+  printTemplateArgumentList(ostream, ArrayRef<TemplateArgumentLoc>(arg),
+                            DefaultPrintPolicy());
   return ostream.str();
 }
 
